@@ -5,11 +5,11 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
 require Exporter;
 
-@ISA = qw(Exporter AutoLoader);
+@ISA = qw(Exporter);
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
-$VERSION = '0.96';
+$VERSION = '1.0';
 
 
 # Preloaded methods go here.
@@ -183,6 +183,12 @@ sub ignore_nonexistant {
     return $self->{ignore_nonexistant};
 }
 
+sub name_changes {
+    my $self=shift;
+    $self->{name_changes_callback}=shift if @_;
+    return $self->{name_changes_callback};
+}
+
 sub TIEHANDLE {
     my $ref=new(@_);
 }
@@ -252,7 +258,9 @@ sub new {
 			 ($object->maxinterval*$object->adjustafter));
     $object->{"debug"}=($params{'debug'} || 0);
     $object->{"tail"}=($params{'tail'} || 0);
+    $object->{"nowait"}=($params{'nowait'} || 0);
     $object->{"maxbuf"}=($params{'maxbuf'} || 16384);
+    $object->{"name_changes_callback"}=($params{'name_changes'} || undef);
     if (defined $params{'reset_tail'}) {
         $object->{"reset_tail"} = $params{'reset_tail'};
     } else {
@@ -267,9 +275,15 @@ sub new {
     if ($object->{"method"} eq "tail") {
 	$object->reset_pointers;
     }
+#    $object->{curpos}=0;        # ADDED 25May01: undef warnings when
+#    $object->{endpos}=0;        #   starting up on a nonexistant file
     return $object;
 }
 
+# Sets position in file when first opened or after that when reset:
+# Sets {endpos} and {curpos} for current {handle} based on {tail}.
+# Sets {tail} to value of {reset_tail}; effect is that first call 
+# uses {tail} and subsequent calls use {reset_tail}.
 sub position {
     my $object=shift;
     $object->{"endpos"}=sysseek($object->{handle},0,SEEK_END);
@@ -310,6 +324,16 @@ sub position {
     $object->{"tail"}=$object->{"reset_tail"};
 }
 
+# Tries to open or reopen the file; failure is an error unless 
+# {ignore_nonexistant} is set. 
+# 
+# For a new file (ie, first time opened) just does some book-keeping 
+# and calls position for initial position setup.  Otherwise does some 
+# checks whether file has been replaced, and if so changes to the new 
+# file.  (Calls position for reset setup).
+#
+# Always updates {lastreset} to current time.
+#
 sub reset_pointers {
     my $object=shift @_;
     $object->{lastreset} = time();
@@ -319,13 +343,28 @@ sub reset_pointers {
     my $oldhandle=$object->{handle};
     my $newhandle=FileHandle->new;
 
-    unless (open($newhandle,"<".$object->input)) {
+    my $newname;
+    if ($oldhandle && $$object{'name_changes_callback'}) {
+	$newname=$$object{'name_changes_callback'}();
+    } else {
+	$newname=$object->input;
+    }
+
+    unless (open($newhandle,"<$newname")) {
 	if ($object->{'ignore_nonexistant'}) {
-	    $object->{'endpos'}=0;
-	    $object->{'curpos'}=0;
+         # If we have an oldhandle, leave endpos and curpos to what they 
+         # were, since oldhandle will still be the "current" handle elsewhere, 
+         # eg, checkpending.  This also allows tailing a file which is removed 
+         # but still being written to.
+            if (!$oldhandle) {
+                $object->{'endpos'}=0;
+                $object->{'curpos'}=0;
+            }
 	    return;
 	}
 	$object->error("Error opening ".$object->input.": $!");
+	$object->{'endpos'}=0 unless defined($object->{'endpos'});
+	$object->{'curpos'}=0 unless defined($object->{'curpos'});
 	return;
     }
     binmode($newhandle);
@@ -342,24 +381,24 @@ sub reset_pointers {
 	# Of course, if the file was removed the same second as when it was
         # last read, and replaced (within that second) with a file of equal
         # length, we're out of luck. I don't see how to fix this.
-	if (($st->mtime<int($object->{lastread}))) {
- 	    $object->{lastread} = $st->mtime; # repair fraction trouble
-	    return;
-	} elsif (($st->mtime==int($object->{lastread})) && 
-		 $st->size==$object->{"curpos"}) {
- 	    $object->{lastread} = $st->mtime; # repair fraction trouble
-	    return;
-
+	if ($st->mtime<=int($object->{'lastread'})) {
+	    if ($st->size==$object->{"curpos"}) {
+		$object->{lastread} = $st->mtime; 
+		return;
+	    } else { 
+		# will continue further to reset
+	    }
+	} else {
 	}
 	$object->{handle}=$newhandle;
 	$object->position;
-#	$object->{lastread} = $st->mtime; # repair fraction trouble
+	$object->{lastread} = $st->mtime;
 	close($oldhandle);
     } else {                  # This is the first time we are opening this file
 	$st=stat($newhandle);
 	$object->{handle}=$newhandle;
 	$object->position;
-#	$object->{lastread}=$st->mtime; # for better estimate on initial read
+	$object->{lastread}=$st->mtime; # for better estimate on initial read
     }
     
 }
@@ -379,9 +418,9 @@ sub checkpending {
    
    $object->{"endpos"}=sysseek($object->{handle},0,SEEK_END);
    if ($object->{"endpos"}<$object->{curpos}) {  # file was truncated
-       $object->{curpos}=sysseek($object->{handle},0,SEEK_SET);
+       $object->position;
    } elsif (($object->{curpos}==$object->{"endpos"}) 
-	       && (time()-$object->{lastreset})>$object->{'resetafter'}) {
+	       && (time()-$object->{lastread})>$object->{'resetafter'}) {
        $object->reset_pointers;
        $object->{"endpos"}=sysseek($object->{handle},0,SEEK_END);
    }
@@ -477,6 +516,11 @@ sub readin {
     while ($nlen>0) {
 	$len=sysread($object->{handle},$object->{"buffer"},
 		     $nlen,length($object->{"buffer"}));
+	last if $len==0; # Some busy filesystems return 0 sometimes, 
+                             # and never give anything more from then on if 
+                             # you don't give them time to rest. This return 
+                             # allows File::Tail to use the usual exponential 
+                             # backoff.
 	$nlen=$nlen-$len;
     }
     $object->{curpos}=sysseek($object->{handle},0,SEEK_CUR);
@@ -484,7 +528,8 @@ sub readin {
     $crs=$object->{"buffer"}=~tr/\n//;
     
     if ($crs) {
-	my $tmp=time; 
+	my $tmp=time;
+	$object->{lastread}=$tmp if $object->{lastread}>$tmp; #???
 	$object->interval(($tmp-($object->{lastread}))/$crs);
 	$object->{lastread}=$tmp;
     }
@@ -495,7 +540,7 @@ sub read {
     my $object=shift @_;
     my $len;
     my $pending=$object->{"endpos"}-$object->{"curpos"};
-    my $crs=$object->{"buffer"}=~tr/\n//;
+    my $crs=$object->{"buffer"}=~m/\n/;
     while (!$pending && !$crs) {
 	$object->{"sleepcount"}=0;
 	while ($object->predict) {
@@ -506,10 +551,10 @@ sub read {
 		    return "";
 		}
 	    }
-	    sleep($object->interval);
+	    sleep($object->interval) if ($object->interval>0);
 	}
 	$pending=$object->{"endpos"}-$object->{"curpos"};
-	$crs=$object->{"buffer"}=~tr/\n//;
+	$crs=$object->{"buffer"}=~m/\n/;
     }
     
     if (!length($object->{"buffer"}) || index($object->{"buffer"},"\n")<0) {
@@ -540,7 +585,7 @@ __END__
 
 =head1 NAME
 
-File::Tail - Perl extension for reading from continously updated files
+File::Tail - Perl extension for reading from continuously updated files
 
 =head1 SYNOPSIS
 
@@ -559,12 +604,11 @@ File::Tail - Perl extension for reading from continously updated files
 OR, you could use tie (additional parameters can be passed with the name, or 
 can be set using $ref):
 
-    use File::Tail;
-    my $ref=tie *FH,"File::Tail",(name=>$name);
-    while (<FH>) {
-        print "$_";
-    }
-}
+  use File::Tail;
+  my $ref=tie *FH,"File::Tail",(name=>$name);
+  while (<FH>) {
+      print "$_";
+  }
 
 Note that the above script will never exit. If there is nothing being written
 to the file, it will simply block.
@@ -582,7 +626,7 @@ in the ditribution.
 =head1 DESCRIPTION
 
 The primary purpose of File::Tail is reading and analysing log files while
-they are being written, which is especialy usefull if you are monitoring
+they are being written, which is especialy useful if you are monitoring
 the logging process with a tool like Tobias Oetiker's MRTG.
 
 The module tries very hard NOT to "busy-wait" on a file that has little 
@@ -612,7 +656,7 @@ the methods.
 
 =head2 new ([ ARGS ]) 
 
-Creates a C<File::Tail>. If it has only one paramter, it is assumed to 
+Creates a C<File::Tail>. If it has only one parameter, it is assumed to
 be the filename. If the open fails, the module performs a croak. I
 am currently looking for a way to set $! and return undef. 
 
@@ -653,7 +697,7 @@ adjustafter*maxinterval.
 =item maxbuf
 
 The maximum size of the internal buffer. When File::Tail
-suddenly found an enormous ammount information in the file
+suddenly found an enormous ammount of information in the file
 (for instance if the retry parameters were set to very
 infrequent checking and the file was rotated), File::Tail
 sometimes slurped way too much file into memory.  This sets
@@ -661,6 +705,10 @@ the maximum size of File::Tail's buffer.
 
 Default value is 16384 (bytes).
 
+A large internal buffer may result in worse performance (as well as
+increased memory usage), since File::Tail will have to do more work
+processing the internal buffer.
+
 =item nowait
 
 Does not block on read, but returns an empty string if there is nothing
@@ -671,39 +719,49 @@ If you want to read tails from multiple files, use select.
 
 =item ignore_nonexistant
 
-    Do not complain if the file doesn't exist when it is first 
+Do not complain if the file doesn't exist when it is first
 opened or when it is to be reopened. (File may be reopened after 
 resetafter seconds have passed since last data was found.)
 
 =item tail
 
-    When first started, read and return C<n> lines from the file. 
+When first started, read and return C<n> lines from the file.
 If C<n> is zero, start at the end of file. If C<n> is negative, 
 return the whole file.
 
-    Default is C<0>.
+Default is C<0>.
 
 =item reset_tail
 
-    Same as tail, but applies after reset. (i.e. after the
-file has been automaticaly closed and reopened). Defaults to
+Same as tail, but applies after reset. (i.e. after the
+file has been automatically closed and reopened). Defaults to
 C<-1>, i.e. does not skip any information present in the
 file when it first checks it.
 
-   Why would you want it otherwise? I've seen files which
+Why would you want it otherwise? I've seen files which
 have been cycled like this:
 
-   grep -v lastmonth log >newlog 
-   mv log archive/lastmonth 
-   mv newlog log 
-   kill -HUP logger 
-
+  grep -v lastmonth log >newlog
+  mv log archive/lastmonth
+  mv newlog log
+  kill -HUP logger
 
 Obviously, if this happens and you have reset_tail set to
-c<-1>, you will suddenly get a whole bunch of lines - lines
+C<-1>, you will suddenly get a whole bunch of lines - lines
 you already saw. So in this case, reset_tail should probably
 be set to a small positive number or even C<0>.
 
+=item name_changes
+
+Some logging systems change the name of the file 
+they are writing to, sometimes to include a date, sometimes a 
+sequence number, sometimes other, even more bizarre changes. 
+
+Instead of trying to implement various clever detection methods, 
+File::Tail will call the code reference defined in name_changes. The code reference should return the string which is the new name of the file to try opening.
+
+Note that if the file does not exist, File::Tail will report a fatal error (unless ignore_nonexistant has also been specified). 
+
 =item debug
 
 Set to nonzero if you want to see more about the inner workings of
@@ -717,9 +775,9 @@ is called with the error string as a parameter, an array with a code
 reference as the first parameter and other parameters to be passed to 
 handler subroutine, or one of the words:
 
-return  - ignore any error (just put error message in errmsg).
-warn    - output the error message but continue
-die     - display error message and exit
+ return  - ignore any error (just put error message in errmsg).
+ warn    - output the error message but continue
+ die     - display error message and exit
 
 Default is die.
 
@@ -739,13 +797,31 @@ input on normal filehandles and File::Tail filehandles. Of course, you may
 use it to simply read from more than one File::Tail filehandle at a time.
 
 
-For an example of usage, look at the script select_demo.
-
 Basicaly, you call File::Tail::select just as you would normal select,
 with fields for rbits, wbits and ebits, as well as a timeout, however, you 
 can tack any number of File::Tail objects (not File::Tail filehandles!) 
 to the end. 
 
+Usage example:
+
+ foreach (@ARGV) {
+     push(@files,File::Tail->new(name=>"$_",debug=>$debug));
+ }
+ while (1) {
+   ($nfound,$timeleft,@pending)=
+             File::Tail::select(undef,undef,undef,$timeout,@files);
+   unless ($nfound) {
+     # timeout - do something else here, if you need to
+   } else {
+     foreach (@pending) {
+        print $_->{"input"}." (".localtime(time).") ".$_->read;
+   }
+ }
+
+ #
+ # There is a more elaborate example in select_demo in the distribution.
+ #
+
 When you do this, File::Tail's select emulates normal select, with two 
 exceptions: 
 
@@ -763,7 +839,10 @@ or you can check each individual object with $object->predict. This returns
 the ammount of time (in fractional seconds) after which the handle expects
 input. If it returns 0, there is input waiting. There is no guarantee that
 there will be input waiting after the returned number of seconds has passed.
-However, File::Tail won't do any I/O on the file until that time has passed. 
+However, File::Tail won't do any I/O on the file until that time has passed.
+Note that the value of $timeleft may or may not be correct - that depends on 
+the underlying operating system (and it's select), so you're better off NOT
+relying on it. 
 
 Also note, if you are determining which files are ready for input by calling
 each individual predict, the C<$nfound> value may be invalid, because one
@@ -772,8 +851,11 @@ has returned and the time when you checked it.
 
 =head1 TO BE DONE
 
-Planned for 0.95-1.0: Using $/ instead of \n to
+Planned for 1.0: Using $/ instead of \n to
 separate "lines" (which should make it possible to read wtmp type files).
+Except that I discovered I have no need for that enhancement If you do, 
+feel free to send me the patches and I'll apply them - if I feel they don't
+add too much processing time.
 
 =head1 AUTHOR
 
